==> intclock-2.12.tbz: lib/perl5/site_perl/5.8.8/Intclock/TZ.pm <==
# $Id: TZ.pm 2902 2007-08-08 19:04:08Z verthezp $
# $URL: file:///var/lib/svn/intclock/tags/R2_12/lib/Intclock/TZ.pm $
#___________________________________________________________________
#
# Timezone handling routines for intclock
#___________________________________________________________________

# Copyright (C) 2004-2007 by Peter Verthez

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

# Written by Peter Verthez, <Peter.Verthez@advalvas.be>.
# Based on Jens-Ulrik Petersen's hsclock

package Intclock::TZ;

use strict;
use diagnostics;
use POSIX;
use vars qw($zonedir @zonedirs $countryfile $zonefile
	    %countryhash %zonehash
	    $cldrterrfile %cldrterrhash
	    $cldrzonefile %cldrzonehash
	    %invzonehash);

BEGIN {
  @zonedirs = ( "/usr/share/zoneinfo",
		"/usr/lib/zoneinfo",
		"/usr/local/share/zoneinfo",
		"/usr/local/lib/zoneinfo",
		"/etc/zoneinfo",
		);
  
  $countryfile = "iso3166.tab";
  $zonefile    = "zone.tab";
  $cldrterrfile = "territories.dat";
  $cldrzonefile = "zones.dat";
}

sub init {
  my ($language) = @_;
  set_timezone();
  _get_zone_dir();
  _get_countries();
  if (defined $language) {
    _get_cldr();
  }
  _get_zones($language);
}

sub _get_zone_dir {
  # some heuristics:
  # - if TZDIR is set and the directory exists, use it
  if (exists $ENV{"TZDIR"} and -d $ENV{"TZDIR"}) {
    $zonedir = $ENV{"TZDIR"};
  }
  # - if 'tzselect' exists, try to find the default TZDIR there
  if (!$zonedir or ! -d $zonedir) {
    my $tzselect=`which tzselect`;
    chomp $tzselect;
    if (-r $tzselect) {
      my $line = `grep "TZDIR=" $tzselect`;
      if ($line =~ /^: \$\{TZDIR=(.*?)\}/) {
	$zonedir = $1;
      }
    }
  }
  # - otherwise, use the fixed list of directories above to search
  if (!$zonedir or ! -d $zonedir) {
    foreach my $dir (@zonedirs) {
      if (-d $dir) {
	$zonedir = $dir;
	last;
      }
    }
  }
  ($zonedir and -d $zonedir) or die "No zoneinfo directory found\n";
}

sub _get_countries {
  open INPUT, "/usr/local/share/iso-codes/iso_3166.tab" or die "Can't read country file\n";
  while (<INPUT>) {
    next if /^\#/;
    next if /^\s*$/;
    my ($code, $name) = split /\t/;
    chomp $name;
    $countryhash{$code} = $name;
  }
  close INPUT;
}

sub _get_cldr_sub {
  my ($file, $hashref, $currlang) = @_;
  open INPUT, "<:utf8", $file or die "Can't read CLDR data file '$file'\n";
  my $lang;
  while (<INPUT>) {
    my $line = $_;
    next if /^\#/;
    next if /^\s*$/;
    if ($line =~ /^\[LANG=(.*?)\]/) {
      $lang = $1;
    }
    elsif ($lang and (index($currlang, $lang) == 0)
	   and $line =~ /^\s*(.+?)=(.*)$/) {
      my ($code, $name) = ($1, $2);
      $name =~ s/\s*$//;
      $name =~ s/\&\#x([A-Z0-9]+);/chr(hex($1))/eg;
      if ($name !~ /^\s*$/) {
	$hashref->{$code} = $name;
      }
    }
  }
  close INPUT;
}

sub _get_cldr {
  my $curr_lang = $ENV{"LANG"};
  if ($curr_lang) {
    my $datadir = Intclock::Config::get_datadir();
    _get_cldr_sub("$datadir/$cldrterrfile", \%cldrterrhash, $curr_lang);
    _get_cldr_sub("$datadir/$cldrzonefile", \%cldrzonehash, $curr_lang);
  }
}

sub _get_zones {
  my ($language) = @_;
  open INPUT, "$zonedir/$zonefile" or die "Can't read zone file\n";
  while (<INPUT>) {
    next if /^\#/;
    next if /^\s*$/;
    my ($country, $coord, $tz, $comments) = split /\t/;
    chomp $tz;
    my $full_tz = $tz;
    $comments ||= "";
    chomp $comments;
    $tz =~ s|^([^/]+)/||;
    my $continent = $1;

    $coord =~ s/^(.*?)([\-\+]\d+)$/$2/;
    my $longitude = substr($coord, 0, 6);

    my %data = ("comments"  => $comments,
		"long"      => $longitude,
		"continent" => $continent);

    my $full_continent = $continent;
    if ($continent =~ /^Arctic|Atlantic|Indian|Pacific$/) {
      $full_continent = "$continent Ocean";
    }

    my $l_cont;
    if (defined $language) {
      eval {
	$l_cont = $language->maketext($full_continent);
      };
      if ($@) {
	print "WARNING: '$full_continent' not translated\n";
	$l_cont = $full_continent;
      }
    }
    else {
      $l_cont = $full_continent;
    }

    my $l_country = $countryhash{$country};
    if (defined $language and exists $cldrterrhash{$country}) {
      $l_country = $cldrterrhash{$country};
    }

    my $l_tz = $tz;
    if (defined $language and exists $cldrzonehash{$full_tz}) {
      $l_tz = $cldrzonehash{$full_tz};
      $data{"translated_zone"} = $l_tz;
    }
    else {
      $l_tz =~ s|^.*?/||;
      $l_tz =~ s/_/ /g;
    }
    $zonehash{$l_cont}->{$l_country}->{$tz} = \%data;
    $invzonehash{$full_tz} = [ $l_country, $l_tz,
			       $zonehash{$l_cont}->{$l_country}];
  }
  close INPUT;
  undef %countryhash;
  undef %cldrterrhash;
}

sub zone_info {
  return (\%zonehash, \%invzonehash);
}

sub exists {
  my ($tz) = @_;
  return (-f "$zonedir/$tz");
}

sub set_timezone {
  my ($tz) = @_;
  $ENV{"TZ"} = $tz if defined $tz;
  POSIX::tzset();
}

sub get_timezone_names {
  my ($tz) = @_;
  set_timezone($tz) if defined $tz;
  return POSIX::tzname();
}

sub in_timezone {
  my ($time, $timezone) = @_;
  set_timezone($timezone);
  my @localtime = localtime($time);
  return \@localtime;
}

1;

==> intclock-2.12_5.tbz: lib/perl5/site_perl/5.10.1/Intclock/TZ.pm <==
# $Id: TZ.pm 2902 2007-08-08 19:04:08Z verthezp $
# $URL: file:///var/lib/svn/intclock/tags/R2_12/lib/Intclock/TZ.pm $
#___________________________________________________________________
#
# Timezone handling routines for intclock
#___________________________________________________________________

# Copyright (C) 2004-2007 by Peter Verthez

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

# Written by Peter Verthez, <Peter.Verthez@advalvas.be>.
# Based on Jens-Ulrik Petersen's hsclock

package Intclock::TZ;

use strict;
use diagnostics;
use POSIX;
use vars qw($zonedir @zonedirs $countryfile $zonefile
	    %countryhash %zonehash
	    $cldrterrfile %cldrterrhash
	    $cldrzonefile %cldrzonehash
	    %invzonehash);

BEGIN {
  @zonedirs = ( "/usr/share/zoneinfo",
		"/usr/lib/zoneinfo",
		"/usr/local/share/zoneinfo",
		"/usr/local/lib/zoneinfo",
		"/etc/zoneinfo",
		);
  
  $countryfile = "iso3166.tab";
  $zonefile    = "zone.tab";
  $cldrterrfile = "territories.dat";
  $cldrzonefile = "zones.dat";
}

sub init {
  my ($language) = @_;
  set_timezone();
  _get_zone_dir();
  _get_countries();
  if (defined $language) {
    _get_cldr();
  }
  _get_zones($language);
}

sub _get_zone_dir {
  # some heuristics:
  # - if TZDIR is set and the directory exists, use it
  if (exists $ENV{"TZDIR"} and -d $ENV{"TZDIR"}) {
    $zonedir = $ENV{"TZDIR"};
  }
  # - if 'tzselect' exists, try to find the default TZDIR there
  if (!$zonedir or ! -d $zonedir) {
    my $tzselect=`which tzselect`;
    chomp $tzselect;
    if (-r $tzselect) {
      my $line = `grep "TZDIR=" $tzselect`;
      if ($line =~ /^: \$\{TZDIR=(.*?)\}/) {
	$zonedir = $1;
      }
    }
  }
  # - otherwise, use the fixed list of directories above to search
  if (!$zonedir or ! -d $zonedir) {
    foreach my $dir (@zonedirs) {
      if (-d $dir) {
	$zonedir = $dir;
	last;
      }
    }
  }
  ($zonedir and -d $zonedir) or die "No zoneinfo directory found\n";
}

sub _get_countries {
  open INPUT, "/usr/share/misc/iso3166" or die "Can't read country file\n";
  while (<INPUT>) {
    next if /^\#/;
    next if /^\s*$/;
    my ($code, $name) = split /\t/;
    chomp $name;
    $countryhash{$code} = $name;
  }
  close INPUT;
}

sub _get_cldr_sub {
  my ($file, $hashref, $currlang) = @_;
  open INPUT, "<:utf8", $file or die "Can't read CLDR data file '$file'\n";
  my $lang;
  while (<INPUT>) {
    my $line = $_;
    next if /^\#/;
    next if /^\s*$/;
    if ($line =~ /^\[LANG=(.*?)\]/) {
      $lang = $1;
    }
    elsif ($lang and (index($currlang, $lang) == 0)
	   and $line =~ /^\s*(.+?)=(.*)$/) {
      my ($code, $name) = ($1, $2);
      $name =~ s/\s*$//;
      $name =~ s/\&\#x([A-Z0-9]+);/chr(hex($1))/eg;
      if ($name !~ /^\s*$/) {
	$hashref->{$code} = $name;
      }
    }
  }
  close INPUT;
}

sub _get_cldr {
  my $curr_lang = $ENV{"LANG"};
  if ($curr_lang) {
    my $datadir = Intclock::Config::get_datadir();
    _get_cldr_sub("$datadir/$cldrterrfile", \%cldrterrhash, $curr_lang);
    _get_cldr_sub("$datadir/$cldrzonefile", \%cldrzonehash, $curr_lang);
  }
}

sub _get_zones {
  my ($language) = @_;
  open INPUT, "$zonedir/$zonefile" or die "Can't read zone file\n";
  while (<INPUT>) {
    next if /^\#/;
    next if /^\s*$/;
    my ($country, $coord, $tz, $comments) = split /\t/;
    chomp $tz;
    my $full_tz = $tz;
    $comments ||= "";
    chomp $comments;
    $tz =~ s|^([^/]+)/||;
    my $continent = $1;

    $coord =~ s/^(.*?)([\-\+]\d+)$/$2/;
    my $longitude = substr($coord, 0, 6);

    my %data = ("comments"  => $comments,
		"long"      => $longitude,
		"continent" => $continent);

    my $full_continent = $continent;
    if ($continent =~ /^Arctic|Atlantic|Indian|Pacific$/) {
      $full_continent = "$continent Ocean";
    }

    my $l_cont;
    if (defined $language) {
      eval {
	$l_cont = $language->maketext($full_continent);
      };
      if ($@) {
	print "WARNING: '$full_continent' not translated\n";
	$l_cont = $full_continent;
      }
    }
    else {
      $l_cont = $full_continent;
    }

    my $l_country = $countryhash{$country};
    if (defined $language and exists $cldrterrhash{$country}) {
      $l_country = $cldrterrhash{$country};
    }

    my $l_tz = $tz;
    if (defined $language and exists $cldrzonehash{$full_tz}) {
      $l_tz = $cldrzonehash{$full_tz};
      $data{"translated_zone"} = $l_tz;
    }
    else {
      $l_tz =~ s|^.*?/||;
      $l_tz =~ s/_/ /g;
    }
    $zonehash{$l_cont}->{$l_country}->{$tz} = \%data;
    $invzonehash{$full_tz} = [ $l_country, $l_tz,
			       $zonehash{$l_cont}->{$l_country}];
  }
  close INPUT;
  undef %countryhash;
  undef %cldrterrhash;
}

sub zone_info {
  return (\%zonehash, \%invzonehash);
}

sub exists {
  my ($tz) = @_;
  return (-f "$zonedir/$tz");
}

sub set_timezone {
  my ($tz) = @_;
  $ENV{"TZ"} = $tz if defined $tz;
  POSIX::tzset();
}

sub get_timezone_names {
  my ($tz) = @_;
  set_timezone($tz) if defined $tz;
  return POSIX::tzname();
}

sub in_timezone {
  my ($time, $timezone) = @_;
  set_timezone($timezone);
  my @localtime = localtime($time);
  return \@localtime;
}

1;
